import inspect
from collections.abc import Sequence
from copy import deepcopy
from typing import (
    TYPE_CHECKING,
    Annotated,
    Any,
    Callable,
    Optional,
    TypeVar,
)

from typing_extensions import (
    ParamSpec,
    get_args,
    get_origin,
)

from fast_depends.dependencies.model import Dependant
from fast_depends.library import CustomField
from fast_depends.library.serializer import OptionItem, Serializer, SerializerProto
from fast_depends.utils import (
    get_typed_signature,
    is_async_gen_callable,
    is_coroutine_callable,
    is_gen_callable,
)

from .model import CallModel

if TYPE_CHECKING:
    from fast_depends.dependencies.provider import Key, Provider


CUSTOM_ANNOTATIONS = (
    Dependant,
    CustomField,
)


P = ParamSpec("P")
T = TypeVar("T")


def build_call_model(
    call: Callable[..., Any],
    *,
    dependency_provider: "Provider",
    use_cache: bool = True,
    is_sync: Optional[bool] = None,
    extra_dependencies: Sequence[Dependant] = (),
    serializer_cls: Optional["SerializerProto"] = None,
    serialize_result: bool = True,
) -> CallModel:
    if hasattr(call, "_fastdepends_call_") and not hasattr(call, "_mock_name"):
        call = call._fastdepends_call_

    name = getattr(inspect.unwrap(call), "__name__", type(call).__name__)

    is_call_async = is_coroutine_callable(call) or is_async_gen_callable(call)
    if is_sync is None:
        is_sync = not is_call_async
    else:
        assert not (is_sync and is_call_async), (
            f"You cannot use async dependency `{name}` at sync main"
        )

    typed_params, return_annotation = get_typed_signature(call)
    if (is_call_generator := is_gen_callable(call) or is_async_gen_callable(call)) and (
        return_args := get_args(return_annotation)
    ):
        return_annotation = return_args[0]

    if not serialize_result:
        return_annotation = inspect.Parameter.empty

    class_fields: list[OptionItem] = []
    dependencies: dict[str, Key] = {}
    custom_fields: dict[str, CustomField] = {}
    positional_args: list[str] = []
    keyword_args: list[str] = []
    args_name: Optional[str] = None
    kwargs_name: Optional[str] = None

    for param_name, param in typed_params.parameters.items():
        dep: Optional[Dependant] = None
        custom: Optional[CustomField] = None

        if param.annotation is inspect.Parameter.empty:
            annotation = Any

        elif get_origin(param.annotation) is Annotated:
            annotated_args = get_args(param.annotation)
            type_annotation = annotated_args[0]

            custom_annotations = []
            regular_annotations = []
            for arg in annotated_args[1:]:
                if isinstance(arg, CUSTOM_ANNOTATIONS):
                    custom_annotations.append(arg)
                else:
                    regular_annotations.append(arg)

            assert len(custom_annotations) <= 1, (
                f"Cannot specify multiple `Annotated` Custom arguments for `{param_name}`!"
            )

            next_custom = next(iter(custom_annotations), None)
            if next_custom is not None:
                if isinstance(next_custom, Dependant):
                    dep = next_custom
                elif isinstance(next_custom, CustomField):
                    custom = deepcopy(next_custom)
                else:  # pragma: no cover
                    raise AssertionError("unreachable")

                if regular_annotations:
                    annotation = param.annotation
                else:
                    annotation = type_annotation
            else:
                annotation = param.annotation
        else:
            annotation = param.annotation

        default: Any
        if param.kind is inspect.Parameter.VAR_POSITIONAL:
            default = ()
        elif param.kind is inspect.Parameter.VAR_KEYWORD:
            default = {}
        else:
            default = param.default

        if isinstance(default, Dependant):
            assert not dep, (
                "You can not use `Depends` with `Annotated` and default both"
            )
            dep, default = default, Ellipsis

        elif isinstance(default, CustomField):
            assert not custom, (
                "You can not use `CustomField` with `Annotated` and default both"
            )
            custom, default = default, Ellipsis

        elif not dep and not custom:
            class_fields.append(
                OptionItem(
                    field_name=param_name,
                    field_type=annotation,
                    default_value=...
                    if default is inspect.Parameter.empty
                    else default,
                    kind=param.kind,
                )
            )

        if dep:
            dependency = build_call_model(
                dep.dependency,
                dependency_provider=dependency_provider,
                use_cache=dep.use_cache,
                is_sync=is_sync,
                serializer_cls=serializer_cls,
                serialize_result=dep.cast_result,
            )

            key = dependency_provider.add_dependant(dependency)

            overrided_dependency = dependency_provider.get_dependant(key)

            assert not (is_sync and is_coroutine_callable(overrided_dependency.call)), (
                f"You cannot use async dependency `{overrided_dependency.call_name}` at sync main"
            )

            dependencies[param_name] = key

            if not dep.cast:
                annotation = Any

            class_fields.append(
                OptionItem(
                    field_name=param_name,
                    field_type=annotation,
                    source=dep,
                    kind=param.kind,
                )
            )

            keyword_args.append(param_name)

        elif custom:
            assert not (is_sync and is_coroutine_callable(custom.use)), (
                f"You cannot use async custom field `{type(custom).__name__}` at sync `{name}`"
            )

            custom.set_param_name(param_name)
            custom_fields[param_name] = custom

            if not custom.cast:
                annotation = Any

            if custom.required:
                class_fields.append(
                    OptionItem(
                        field_name=param_name,
                        field_type=annotation,
                        default_value=...,
                        source=custom,
                        kind=param.kind,
                    )
                )

            else:
                class_fields.append(
                    OptionItem(
                        field_name=param_name,
                        field_type=Optional[annotation],
                        default_value=None
                        if (default is inspect.Parameter.empty or default is Ellipsis)
                        else default,
                        source=custom,
                        kind=param.kind,
                    )
                )

            keyword_args.append(param_name)

        else:
            if param.kind is param.KEYWORD_ONLY:
                keyword_args.append(param_name)
            elif param.kind is param.VAR_KEYWORD:
                kwargs_name = param_name
            elif param.kind is param.VAR_POSITIONAL:
                args_name = param_name
            else:
                positional_args.append(param_name)

    serializer: Optional[Serializer] = None
    if serializer_cls is not None:
        serializer = serializer_cls(
            name=name,
            options=class_fields,
            response_type=return_annotation,
        )

    solved_extra_dependencies: list[Key] = []
    for dep in extra_dependencies:
        dependency = build_call_model(
            dep.dependency,
            dependency_provider=dependency_provider,
            use_cache=dep.use_cache,
            is_sync=is_sync,
            serializer_cls=serializer_cls,
        )

        key = dependency_provider.add_dependant(dependency)

        overrided_dependency = dependency_provider.get_dependant(key)

        assert not (is_sync and is_coroutine_callable(overrided_dependency.call)), (
            f"You cannot use async dependency `{overrided_dependency.call_name}` at sync main"
        )

        solved_extra_dependencies.append(key)

    return CallModel(
        call=call,
        serializer=serializer,
        params=tuple(
            i
            for i in class_fields
            if (i.field_name not in dependencies and i.field_name not in custom_fields)
        ),
        use_cache=use_cache,
        is_async=is_call_async,
        is_generator=is_call_generator,
        dependencies=dependencies,
        custom_fields=custom_fields,
        positional_args=positional_args,
        keyword_args=keyword_args,
        args_name=args_name,
        kwargs_name=kwargs_name,
        extra_dependencies=solved_extra_dependencies,
        dependency_provider=dependency_provider,
    )



from collections.abc import Sequence
from typing import Any

from fast_depends.library.serializer import OptionItem


class FastDependsError(Exception):
    pass


class ValidationError(ValueError, FastDependsError):
    def __init__(
        self,
        *,
        incoming_options: Any,
        locations: Sequence[Any],
        expected: dict[str, OptionItem],
        original_error: Exception,
    ) -> None:
        self.original_error = original_error
        self.incoming_options = incoming_options

        self.error_fields: tuple[OptionItem, ...] = tuple(
            expected[x] for x in locations if x in expected
        )
        if not self.error_fields:
            self.error_fields = tuple(expected.values())

        super().__init__()

    def __str__(self) -> str:
        if isinstance(self.incoming_options, dict):
            content = ", ".join(f"{k}=`{v}`" for k, v in self.incoming_options.items())
        else:
            content = f"`{self.incoming_options}`"

        return (
            "\n    Incoming options: "
            + content
            + "\n    In the following option types error occured:\n    "
            + "\n    ".join(map(str, self.error_fields))
        )



import sys
import typing
from importlib.metadata import version as get_version

__all__ = (
    "ExceptionGroup",
    "evaluate_forwardref",
)


ANYIO_V3 = get_version("anyio").startswith("3.")

if ANYIO_V3:
    from anyio import ExceptionGroup as ExceptionGroup
else:
    if sys.version_info < (3, 11):
        from exceptiongroup import ExceptionGroup as ExceptionGroup
    else:
        ExceptionGroup = ExceptionGroup


def evaluate_forwardref(
    value: typing.Any,
    globalns: typing.Optional[dict[str, typing.Any]] = None,
    localns: typing.Optional[dict[str, typing.Any]] = None,
    type_params: typing.Optional[tuple[typing.Any, ...]] = None,
) -> typing.Any:
    """Behaves like typing._eval_type, except it won't raise an error if a forward reference can't be resolved."""
    if value is None:
        value = NoneType

    elif isinstance(value, str):
        value = _make_forward_ref(value, is_argument=False, is_class=True)

    try:
        return eval_type_backport(value, globalns, localns, type_params=type_params)
    except NameError:
        # the point of this function is to be tolerant to this case
        return value


def eval_type_backport(
    value: typing.Any,
    globalns: typing.Optional[dict[str, typing.Any]] = None,
    localns: typing.Optional[dict[str, typing.Any]] = None,
    type_params: typing.Optional[tuple[typing.Any, ...]] = None,
) -> typing.Any:
    """Like `typing._eval_type`, but falls back to the `eval_type_backport` package if it's
    installed to let older Python versions use newer typing features.
    Specifically, this transforms `X | Y` into `typing.Union[X, Y]`
    and `list[X]` into `typing.List[X]` etc. (for all the types made generic in PEP 585)
    if the original syntax is not supported in the current Python version.
    """
    try:
        if sys.version_info >= (3, 13):
            return typing._eval_type(  # type: ignore
                value, globalns, localns, type_params=type_params
            )

        else:
            return typing._eval_type(  # type: ignore
                value, globalns, localns
            )

    except TypeError as e:
        if not (isinstance(value, typing.ForwardRef) and is_backport_fixable_error(e)):
            raise

        try:
            from eval_type_backport import eval_type_backport as _eval_type_backport

        except ImportError:
            raise TypeError(
                f"You have a type annotation {value.__forward_arg__!r} "
                f"which makes use of newer typing features than are supported in your version of Python. "
                f"To handle this error, you should either remove the use of new syntax "
                f"or install the `eval_type_backport` package."
            ) from e

        else:
            return _eval_type_backport(value, globalns, localns, try_default=False)


def is_backport_fixable_error(e: TypeError) -> bool:
    msg = str(e)
    return msg.startswith("unsupported operand type(s) for |: ") or "' object is not subscriptable" in msg


if sys.version_info < (3, 10):
    NoneType = type(None)
else:
    from types import NoneType as NoneType


if sys.version_info < (3, 9, 8) or (3, 10) <= sys.version_info < (3, 10, 1):
    def _make_forward_ref(
        arg: typing.Any,
        is_argument: bool = True,
        *,
        is_class: bool = False,
    ) -> typing.ForwardRef:
        """Wrapper for ForwardRef that accounts for the `is_class` argument missing in older versions.
        The `module` argument is omitted as it breaks <3.9.8, =3.10.0 and isn't used in the calls below.

        See https://github.com/python/cpython/pull/28560 for some background.
        The backport happened on 3.9.8, see:
        https://github.com/pydantic/pydantic/discussions/6244#discussioncomment-6275458,
        and on 3.10.1 for the 3.10 branch, see:
        https://github.com/pydantic/pydantic/issues/6912

        Implemented as EAFP with memory.
        """
        return typing.ForwardRef(arg, is_argument)

else:
    _make_forward_ref = typing.ForwardRef




from collections.abc import Hashable, Iterator
from contextlib import contextmanager
from typing import TYPE_CHECKING, Any, Callable

from typing_extensions import TypeAlias

from fast_depends.core import build_call_model

if TYPE_CHECKING:
    from fast_depends.core import CallModel


Key: TypeAlias = Hashable


class Provider:
    dependencies: dict[Key, "CallModel"]
    overrides: dict[Key, "CallModel"]

    def __init__(self) -> None:
        self.dependencies = {}
        self.overrides = {}

    def clear(self) -> None:
        self.dependencies = {}
        self.overrides = {}

    def add_dependant(
        self,
        dependant: "CallModel",
    ) -> Key:
        key = self.__get_original_key(dependant.call)
        self.dependencies[key] = dependant
        return key

    def get_dependant(self, key: Key) -> "CallModel":
        return self.overrides.get(key) or self.dependencies[key]

    def override(
        self,
        original: Callable[..., Any],
        override: Callable[..., Any],
    ) -> None:
        key = self.__get_original_key(original)

        override_model = build_call_model(
            override,
            dependency_provider=self,
        )

        if (original_dependant := self.dependencies.get(key)):
            override_model.serializer = original_dependant.serializer

        else:
            self.dependencies[key] = build_call_model(
                original,
                dependency_provider=self,
            )

        self.overrides[key] = override_model

    @contextmanager
    def scope(
        self,
        original: Callable[..., Any],
        override: Callable[..., Any],
    ) -> Iterator[None]:
        self.override(original, override)
        yield
        self.overrides.pop(self.__get_original_key(original), None)

    def __get_original_key(self, original: Callable[..., Any]) -> Key:
        return original



import inspect
import json
from abc import ABC, abstractmethod
from typing import Any, Protocol


class OptionItem:
    __slots__ = (
        "field_name",
        "field_type",
        "default_value",
        "kind",
        "source",
    )

    def __init__(
        self,
        field_name: str,
        field_type: Any,
        source: Any = None,
        default_value: Any = ...,
        kind: inspect._ParameterKind = inspect.Parameter.POSITIONAL_OR_KEYWORD,
    ) -> None:
        self.field_name = field_name
        self.field_type = field_type
        self.default_value = default_value
        self.source = source
        self.kind = kind

    def __repr__(self) -> str:
        type_name = getattr(self.field_type, "__name__", str(self.field_type))
        content = f"{self.field_name}, type=`{type_name}`"
        if self.default_value is not Ellipsis:
            content = f"{content}, default=`{self.default_value}`"
        if self.source:
            content = f"{content}, source=`{self.source}`"
        return f"OptionItem[{content}]"


class Serializer(ABC):
    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ):
        self.name = name
        self.options = {i.field_name: i for i in options}
        self.response_option = {
            "return": OptionItem(field_name="return", field_type=response_type),
        }

    def get_aliases(self) -> tuple[str, ...]:
        return ()

    @abstractmethod
    def __call__(self, call_kwargs: dict[str, Any]) -> dict[str, Any]:
        raise NotImplementedError

    def response(self, value: Any) -> Any:
        return value


class SerializerProto(Protocol):
    def __call__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ) -> Serializer:
        ...

    @staticmethod
    def encode(message: Any) -> bytes:
        return json.dumps(message).encode("utf-8")



import json
from typing import Any, Callable, Optional

from pydantic import BaseModel, create_model
from pydantic.version import VERSION as PYDANTIC_VERSION

__all__ = (
    "BaseModel",
    "create_model",
    "PYDANTIC_V2",
    "get_config_base",
    "ConfigDict",
    "TypeAdapter",
    "PydanticUserError",
)


json_dumps: Callable[..., bytes]
orjson: Any
ujson: Any

try:
    import orjson
except ImportError:
    orjson = None

try:
    import ujson
except ImportError:
    ujson = None

if orjson:
    json_loads = orjson.loads
    json_dumps = orjson.dumps

elif ujson:
    json_loads = ujson.loads

    def json_dumps(*a: Any, **kw: Any) -> bytes:
        return ujson.dumps(*a, **kw).encode()  # type: ignore[no-any-return]

else:
    json_loads = json.loads

    def json_dumps(*a: Any, **kw: Any) -> bytes:
        return json.dumps(*a, **kw).encode()

PYDANTIC_V2 = PYDANTIC_VERSION.startswith("2.")

default_pydantic_config = {"arbitrary_types_allowed": True}

# isort: off
if PYDANTIC_V2:
    from pydantic import ConfigDict, TypeAdapter
    from pydantic.fields import FieldInfo
    from pydantic.errors import PydanticUserError
    from pydantic_core import to_json as dump_json

    def model_schema(model: type[BaseModel]) -> dict[str, Any]:
        return model.model_json_schema()

    def get_config_base(config_data: Optional[ConfigDict] = None) -> ConfigDict:
        return config_data or ConfigDict(**default_pydantic_config)  # type: ignore[typeddict-item]

    def get_aliases(model: type[BaseModel]) -> tuple[str, ...]:
        return tuple(f.alias or name for name, f in get_model_fields(model).items())

    def get_model_fields(model: type[BaseModel]) -> dict[str, FieldInfo]:
        fields: Optional[dict[str, FieldInfo]] = getattr(
            model, "__pydantic_fields__", None
        )

        if fields is not None:
            return fields

        # Deprecated in Pydantic V2.11 to be removed in V3.0.
        return model.model_fields

else:
    from pydantic.config import get_config, ConfigDict, BaseConfig
    from pydantic.fields import ModelField
    from pydantic.json import pydantic_encoder

    TypeAdapter = None
    PydanticUserError = Exception

    def get_config_base(config_data: Optional[ConfigDict] = None) -> type[BaseConfig]:  # type: ignore[misc]
        return get_config(config_data or ConfigDict(**default_pydantic_config))  # type: ignore[typeddict-item]

    def model_schema(model: type[BaseModel]) -> dict[str, Any]:
        return model.schema()

    def get_aliases(model: type[BaseModel]) -> tuple[str, ...]:
        return tuple(f.alias or name for name, f in model.__fields__.items())

    def get_model_fields(model: type[BaseModel]) -> dict[str, ModelField]:
        return model.__fields__

    def dump_json(data: Any) -> bytes:
        return json_dumps(data, default=pydantic_encoder)



from fast_depends.dependencies import Provider
from fast_depends.exceptions import ValidationError
from fast_depends.use import Depends, inject
from fast_depends.use import global_provider as dependency_provider

__all__ = (
    "Depends",
    "ValidationError",
    "Provider",
    "dependency_provider",
    "inject",
)



import inspect
import re
from collections.abc import Iterator, Sequence
from contextlib import contextmanager
from typing import Any, Union

import msgspec

from fast_depends.exceptions import ValidationError
from fast_depends.library.serializer import OptionItem, Serializer, SerializerProto


class MsgSpecSerializer(SerializerProto):
    __slots__ = ("use_fastdepends_errors",)

    def __init__(
        self,
        use_fastdepends_errors: bool = True,
    ) -> None:
        self.use_fastdepends_errors = use_fastdepends_errors

    def __call__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ) -> "Serializer":
        if self.use_fastdepends_errors:
            if response_type is not inspect.Parameter.empty:
                return _MsgSpecWrappedSerializerWithResponse(
                    name=name,
                    options=options,
                    response_type=response_type,
                )

            return _MsgSpecWrappedSerializer(
                name=name,
                options=options,
            )

        if response_type is not inspect.Parameter.empty:
            return _MsgSpecSerializerWithResponse(
                name=name,
                options=options,
                response_type=response_type,
            )

        return _MsgSpecSerializer(
            name=name,
            options=options,
        )

    @staticmethod
    def encode(message: Any) -> bytes:
        if isinstance(message, bytes):
            return message
        return msgspec.json.encode(message)


class _MsgSpecSerializer(Serializer):
    __slots__ = (
        "aliases",
        "model",
        "response_type",
        "name",
        "options",
        "response_option",
    )

    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any = None,
    ):
        model_options: list[Union[str, tuple[str, type], tuple[str, type, Any]]] = []
        aliases = {}
        for i in options:
            if isinstance(
                msgspec.inspect.type_info(i.field_type), msgspec.inspect.CustomType
            ):
                continue

            default_value = i.default_value

            if isinstance(default_value, msgspec._core.Field) and default_value.name:
                aliases[i.field_name] = default_value.name
            else:
                aliases[i.field_name] = i.field_name

            if default_value is Ellipsis:
                model_options.append(
                    (
                        i.field_name,
                        i.field_type,
                    )
                )
            else:
                model_options.append(
                    (
                        i.field_name,
                        i.field_type,
                        default_value,
                    )
                )

        self.aliases = aliases
        self.model = msgspec.defstruct(name, model_options, kw_only=True)
        super().__init__(name=name, options=options, response_type=response_type)

    def get_aliases(self) -> tuple[str, ...]:
        return tuple(self.aliases.values())

    def __call__(self, call_kwargs: dict[str, Any]) -> dict[str, Any]:
        casted_model = msgspec.convert(
            call_kwargs,
            type=self.model,
            strict=False,
            str_keys=True,
        )

        return {
            out_field: getattr(casted_model, out_field, None)
            for out_field in self.aliases.keys()
        }


class _MsgSpecSerializerWithResponse(_MsgSpecSerializer):
    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ):
        super().__init__(name=name, options=options, response_type=response_type)
        self.response_type = response_type

    def response(self, value: Any) -> Any:
        return msgspec.convert(value, type=self.response_type, strict=False)


class _MsgSpecWrappedSerializer(_MsgSpecSerializer):
    def __call__(self, call_kwargs: dict[str, Any]) -> dict[str, Any]:
        with self._try_msgspec(call_kwargs, self.options):
            casted_model = msgspec.convert(
                call_kwargs,
                type=self.model,
                strict=False,
                str_keys=True,
            )

        return {
            out_field: getattr(casted_model, out_field, None)
            for out_field in self.aliases.keys()
        }

    @contextmanager
    def _try_msgspec(
        self,
        call_kwargs: Any,
        options: dict[str, OptionItem],
        locations: Sequence[str] = (),
    ) -> Iterator[None]:
        try:
            yield
        except msgspec.ValidationError as er:
            raise ValidationError(
                incoming_options=call_kwargs,
                expected=options,
                locations=locations or re.findall(r"at `\$\.(.)`", str(er.args)),
                original_error=er,
            ) from er


class _MsgSpecWrappedSerializerWithResponse(_MsgSpecWrappedSerializer):
    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ):
        super().__init__(name=name, options=options, response_type=response_type)
        self.response_type = response_type

    def response(self, value: Any) -> Any:
        with self._try_msgspec(value, self.response_option, ("return",)):
            return msgspec.convert(value, type=self.response_type, strict=False)



import asyncio
import functools
import inspect
from collections.abc import AsyncGenerator, AsyncIterable, Awaitable
from contextlib import (
    AbstractContextManager,
    AsyncExitStack,
    ExitStack,
    asynccontextmanager,
    contextmanager,
)
from typing import (
    TYPE_CHECKING,
    Annotated,
    Any,
    Callable,
    ForwardRef,
    Optional,
    TypeVar,
    Union,
    cast,
)

import anyio
from typing_extensions import (
    ParamSpec,
    get_args,
    get_origin,
)

from fast_depends._compat import evaluate_forwardref

if TYPE_CHECKING:
    from types import FrameType

P = ParamSpec("P")
T = TypeVar("T")


async def run_async(
    func: Union[
        Callable[P, T],
        Callable[P, Awaitable[T]],
    ],
    *args: P.args,
    **kwargs: P.kwargs,
) -> T:
    if is_coroutine_callable(func):
        return await cast(Callable[P, Awaitable[T]], func)(*args, **kwargs)
    else:
        return await run_in_threadpool(cast(Callable[P, T], func), *args, **kwargs)


async def run_in_threadpool(
    func: Callable[P, T],
    *args: P.args,
    **kwargs: P.kwargs,
) -> T:
    if kwargs:
        func = functools.partial(func, **kwargs)
    return await anyio.to_thread.run_sync(func, *args)


async def solve_generator_async(
    *sub_args: Any,
    call: Callable[..., Any],
    stack: AsyncExitStack,
    **sub_values: Any,
) -> Any:
    if is_gen_callable(call):
        cm = contextmanager_in_threadpool(contextmanager(call)(**sub_values))
    elif is_async_gen_callable(call):  # pragma: no branch
        cm = asynccontextmanager(call)(*sub_args, **sub_values)
    return await stack.enter_async_context(cm)


def solve_generator_sync(
    *sub_args: Any,
    call: Callable[..., Any],
    stack: ExitStack,
    **sub_values: Any,
) -> Any:
    cm = contextmanager(call)(*sub_args, **sub_values)
    return stack.enter_context(cm)


def get_typed_signature(call: Callable[..., Any]) -> tuple[inspect.Signature, Any]:
    signature = inspect.signature(call)

    locals = collect_outer_stack_locals()

    # We unwrap call to get the original unwrapped function
    call = inspect.unwrap(call)

    type_params = getattr(call, "__type_params__", ()) or None

    globalns = getattr(call, "__globals__", {})
    typed_params = [
        inspect.Parameter(
            name=param.name,
            kind=param.kind,
            default=param.default,
            annotation=get_typed_annotation(
                param.annotation,
                globalns,
                locals,
                type_params=type_params,
            ),
        )
        for param in signature.parameters.values()
    ]

    return inspect.Signature(typed_params), get_typed_annotation(
        signature.return_annotation,
        globalns,
        locals,
        type_params=type_params,
    )


def collect_outer_stack_locals() -> dict[str, Any]:
    frame = inspect.currentframe()

    frames: list[FrameType] = []
    while frame is not None:
        if "fast_depends" not in frame.f_code.co_filename:
            frames.append(frame)
        frame = frame.f_back

    locals = {}
    for f in frames[::-1]:
        locals.update(f.f_locals)

    return locals


def get_typed_annotation(
    annotation: Any,
    globalns: dict[str, Any],
    locals: dict[str, Any],
    type_params: Optional[tuple[Any, ...]] = None,
) -> Any:
    if isinstance(annotation, str):
        annotation = ForwardRef(annotation)

    if isinstance(annotation, ForwardRef):
        annotation = evaluate_forwardref(
            annotation, globalns, locals, type_params=type_params
        )

    if get_origin(annotation) is Annotated and (args := get_args(annotation)):
        solved_args = [
            get_typed_annotation(x, globalns, locals, type_params=type_params)
            for x in args
        ]

        annotation.__origin__, annotation.__metadata__ = (
            solved_args[0],
            tuple(solved_args[1:]),
        )

    return annotation


@asynccontextmanager
async def contextmanager_in_threadpool(
    cm: AbstractContextManager[T],
) -> AsyncGenerator[T, None]:
    exit_limiter = anyio.CapacityLimiter(1)
    try:
        yield await run_in_threadpool(cm.__enter__)
    except Exception as e:
        ok = bool(
            await anyio.to_thread.run_sync(
                cm.__exit__, type(e), e, None, limiter=exit_limiter
            )
        )
        if not ok:  # pragma: no branch
            raise e
    else:
        await anyio.to_thread.run_sync(
            cm.__exit__, None, None, None, limiter=exit_limiter
        )


def is_gen_callable(call: Callable[..., Any]) -> bool:
    if inspect.isgeneratorfunction(call):
        return True
    dunder_call = getattr(call, "__call__", None)  # noqa: B004
    return inspect.isgeneratorfunction(dunder_call)


def is_async_gen_callable(call: Callable[..., Any]) -> bool:
    if inspect.isasyncgenfunction(call):
        return True
    dunder_call = getattr(call, "__call__", None)  # noqa: B004
    return inspect.isasyncgenfunction(dunder_call)


def is_coroutine_callable(call: Callable[..., Any]) -> bool:
    if inspect.isclass(call):
        return False

    if asyncio.iscoroutinefunction(call):
        return True

    dunder_call = getattr(call, "__call__", None)  # noqa: B004
    return asyncio.iscoroutinefunction(dunder_call)


async def async_map(
    func: Callable[..., T], async_iterable: AsyncIterable[Any]
) -> AsyncIterable[T]:
    async for i in async_iterable:
        yield func(i)



import inspect
from collections.abc import Iterator, Sequence
from contextlib import contextmanager
from itertools import chain
from typing import Any, Callable, Optional

from pydantic import ValidationError as PValidationError

from fast_depends.exceptions import ValidationError
from fast_depends.library.serializer import OptionItem, Serializer, SerializerProto
from fast_depends.pydantic._compat import (
    PYDANTIC_V2,
    BaseModel,
    ConfigDict,
    PydanticUserError,
    TypeAdapter,
    create_model,
    dump_json,
    get_aliases,
    get_config_base,
    get_model_fields,
)


class PydanticSerializer(SerializerProto):
    __slots__ = (
        "pydantic_config",
        "use_fastdepends_errors",
    )

    def __init__(
        self,
        pydantic_config: Optional[ConfigDict] = None,
        use_fastdepends_errors: bool = True,
    ) -> None:
        self.config = pydantic_config
        self.use_fastdepends_errors = use_fastdepends_errors

    def __call__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
    ) -> "_PydanticSerializer":
        if self.use_fastdepends_errors:
            if response_type is not inspect.Parameter.empty:
                return _PydanticWrappedSerializerWithResponse(
                    name=name,
                    options=options,
                    response_type=response_type,
                    pydantic_config=self.config,
                )

            return _PydanticWrappedSerializer(
                name=name,
                options=options,
                pydantic_config=self.config,
            )

        if response_type is not inspect.Parameter.empty:
            return _PydanticSerializerWithResponse(
                name=name,
                options=options,
                response_type=response_type,
                pydantic_config=self.config,
            )

        return _PydanticSerializer(
            name=name,
            options=options,
            pydantic_config=self.config,
        )

    @staticmethod
    def encode(message: Any) -> bytes:
        if isinstance(message, bytes):
            return message
        return dump_json(message)


class _PydanticSerializer(Serializer):
    __slots__ = (
        "model",
        "name",
        "options",
        "config",
        "response_option",
    )

    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any = None,
        pydantic_config: Optional[ConfigDict] = None,
    ):
        class_options: dict[str, Any] = {
            i.field_name: (i.field_type, i.default_value) for i in options
        }

        self.config = get_config_base(pydantic_config)

        self.model = create_model(
            name,
            __config__=self.config,
            **class_options,
        )

        super().__init__(name=name, options=options, response_type=response_type)

    def get_aliases(self) -> tuple[str, ...]:
        return get_aliases(self.model)

    def __call__(self, call_kwargs: dict[str, Any]) -> dict[str, Any]:
        casted_model = self.model(**call_kwargs)

        return {
            i: getattr(casted_model, i) for i in get_model_fields(casted_model).keys()
        }


class _PydanticSerializerWithResponse(_PydanticSerializer):
    __slots__ = ("response_callback",)

    response_callback: Callable[[Any], Any]

    def __init__(
        self,
        *,
        name: str,
        options: list[OptionItem],
        response_type: Any,
        pydantic_config: Optional[ConfigDict] = None,
    ):
        super().__init__(
            name=name,
            options=options,
            response_type=response_type,
            pydantic_config=pydantic_config,
        )

        response_callback: Optional[Callable[[Any], Any]] = None
        try:
            is_model = issubclass(response_type or object, BaseModel)
        except Exception:
            is_model = False

        if is_model:
            if PYDANTIC_V2:
                response_callback = response_type.model_validate
            else:
                response_callback = response_type.validate

        elif PYDANTIC_V2:
            try:
                response_pydantic_type = TypeAdapter(response_type, config=self.config)
            except PydanticUserError:
                response_pydantic_type = TypeAdapter(response_type)
            response_callback = response_pydantic_type.validate_python

        if response_callback is None and not PYDANTIC_V2:
            response_model = create_model(
                "ResponseModel",
                __config__=self.config,
                r=(response_type or Any, ...),
            )

            response_callback = lambda x: response_model(r=x).r

        assert response_callback
        self.response_callback = response_callback

    def response(self, value: Any) -> Any:
        return self.response_callback(value)


class _PydanticWrappedSerializer(_PydanticSerializer):
    def __call__(self, call_kwargs: dict[str, Any]) -> dict[str, Any]:
        with self._try_pydantic(call_kwargs, self.options):
            casted_model = self.model(**call_kwargs)

        return {
            i: getattr(casted_model, i) for i in get_model_fields(casted_model).keys()
        }

    @contextmanager
    def _try_pydantic(
        self,
        call_kwargs: Any,
        options: dict[str, OptionItem],
        locations: Sequence[str] = (),
    ) -> Iterator[None]:
        try:
            yield
        except PValidationError as er:
            raise ValidationError(
                incoming_options=call_kwargs,
                expected=options,
                locations=locations
                or tuple(chain(*(one_error["loc"] for one_error in er.errors()))),
                original_error=er,
            ) from er


class _PydanticWrappedSerializerWithResponse(
    _PydanticWrappedSerializer,
    _PydanticSerializerWithResponse,
):
    def response(self, value: Any) -> Any:
        with self._try_pydantic(value, self.response_option, ("return",)):
            return self.response_callback(value)



from collections.abc import AsyncIterator, Iterator, Sequence
from contextlib import AsyncExitStack, ExitStack
from functools import partial, wraps
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Optional,
    Protocol,
    TypeVar,
    Union,
    cast,
    overload,
)

from typing_extensions import ParamSpec

from fast_depends.core import CallModel, build_call_model
from fast_depends.dependencies import Dependant, Provider
from fast_depends.library.serializer import SerializerProto

SerializerCls: Optional["SerializerProto"] = None

if SerializerCls is None:
    try:
        from fast_depends.pydantic import PydanticSerializer

        SerializerCls = PydanticSerializer()
    except ImportError:
        pass

if SerializerCls is None:
    try:
        from fast_depends.msgspec import MsgSpecSerializer

        SerializerCls = MsgSpecSerializer()
    except ImportError:
        pass


P = ParamSpec("P")
T = TypeVar("T")

if TYPE_CHECKING:
    from fast_depends.library.serializer import SerializerProto

    class InjectWrapper(Protocol[P, T]):
        def __call__(
            self,
            func: Callable[P, T],
            model: Optional[CallModel] = None,
        ) -> Callable[P, T]:
            ...


global_provider = Provider()


def Depends(
    dependency: Callable[..., Any],
    *,
    use_cache: bool = True,
    cast: bool = True,
    cast_result: bool = False,
) -> Any:
    return Dependant(
        dependency=dependency,
        use_cache=use_cache,
        cast=cast,
        cast_result=cast_result,
    )


@overload
def inject(
    func: Callable[P, T],
    *,
    cast: bool = True,
    cast_result: bool = True,
    extra_dependencies: Sequence["Dependant"] = (),
    dependency_provider: Optional["Provider"] = None,
    wrap_model: Callable[["CallModel"], "CallModel"] = lambda x: x,
    serializer_cls: Optional["SerializerProto"] = SerializerCls,
    **call_extra: Any,
) -> Callable[P, T]:
    ...


@overload
def inject(
    func: None = None,
    *,
    cast: bool = True,
    cast_result: bool = True,
    extra_dependencies: Sequence["Dependant"] = (),
    dependency_provider: Optional["Provider"] = None,
    wrap_model: Callable[["CallModel"], "CallModel"] = lambda x: x,
    serializer_cls: Optional["SerializerProto"] = SerializerCls,
    **call_extra: Any,
) -> "InjectWrapper[P, T]":
    ...


def inject(
    func: Optional[Callable[P, T]] = None,
    *,
    cast: bool = True,
    cast_result: bool = True,
    extra_dependencies: Sequence[Dependant] = (),
    dependency_provider: Optional["Provider"] = None,
    wrap_model: Callable[["CallModel"], "CallModel"] = lambda x: x,
    serializer_cls: Optional["SerializerProto"] = SerializerCls,
    **call_extra: Any,
) -> Union[Callable[P, T], "InjectWrapper[P, T]"]:
    if dependency_provider is None:
        dependency_provider = global_provider

    if not cast:
        serializer_cls = None

    decorator = _wrap_inject(
        dependency_provider=dependency_provider,
        wrap_model=wrap_model,
        extra_dependencies=extra_dependencies,
        serializer_cls=serializer_cls,
        cast_result=cast_result,
        **call_extra,
    )

    if func is None:
        return decorator
    return decorator(func)


def _wrap_inject(
    *,
    dependency_provider: "Provider",
    wrap_model: Callable[["CallModel"], "CallModel"],
    extra_dependencies: Sequence[Dependant],
    serializer_cls: Optional["SerializerProto"],
    cast_result: bool,
    **call_extra: Any,
) -> "InjectWrapper[P, T]":
    def func_wrapper(
        func: Callable[P, T],
        model: Optional["CallModel"] = None,
    ) -> Callable[P, T]:
        if model is None:
            real_model = wrap_model(
                build_call_model(
                    call=func,
                    extra_dependencies=extra_dependencies,
                    dependency_provider=dependency_provider,
                    serializer_cls=serializer_cls,
                    serialize_result=cast_result,
                )
            )
        else:
            real_model = model

        if real_model.is_async:
            injected_wrapper: Callable[P, T]

            if real_model.is_generator:
                injected_wrapper = partial(  # type: ignore[assignment]
                    solve_async_gen,
                    real_model,
                )

            else:

                async def injected_wrapper(*args: P.args, **kwargs: P.kwargs) -> T:  # type: ignore[misc]
                    async with AsyncExitStack() as stack:
                        return await real_model.asolve(  # type: ignore[no-any-return]
                            *args,
                            stack=stack,
                            cache_dependencies={},
                            nested=False,
                            **(call_extra | kwargs),
                        )

                    raise AssertionError("unreachable")

        else:
            if real_model.is_generator:
                injected_wrapper = partial(  # type: ignore[assignment]
                    solve_gen,
                    real_model,
                )

            else:

                def injected_wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
                    with ExitStack() as stack:
                        return real_model.solve(  # type: ignore[no-any-return]
                            *args,
                            stack=stack,
                            cache_dependencies={},
                            nested=False,
                            **(call_extra | kwargs),
                        )

                    raise AssertionError("unreachable")

        injected_wrapper._fastdepends_call_ = real_model.call  # type: ignore[attr-defined]
        return wraps(func)(injected_wrapper)

    return func_wrapper


class solve_async_gen:
    _iter: Optional[AsyncIterator[Any]] = None

    def __init__(
        self,
        model: "CallModel",
        *args: Any,
        **kwargs: Any,
    ):
        self.call = model
        self.args = args
        self.kwargs = kwargs

    def __aiter__(self) -> "solve_async_gen":
        self._iter = None
        self.stack = AsyncExitStack()
        return self

    async def __anext__(self) -> Any:
        if self._iter is None:
            stack = self.stack = AsyncExitStack()
            await self.stack.__aenter__()
            self._iter = cast(
                AsyncIterator[Any],
                (
                    await self.call.asolve(
                        *self.args,
                        stack=stack,
                        cache_dependencies={},
                        nested=False,
                        **self.kwargs,
                    )
                ).__aiter__(),
            )

        try:
            r = await self._iter.__anext__()
        except StopAsyncIteration:
            await self.stack.__aexit__(None, None, None)
            raise
        else:
            return r


class solve_gen:
    _iter: Optional[Iterator[Any]] = None

    def __init__(
        self,
        model: "CallModel",
        *args: Any,
        **kwargs: Any,
    ):
        self.call = model
        self.args = args
        self.kwargs = kwargs

    def __iter__(self) -> "solve_gen":
        self._iter = None
        self.stack = ExitStack()
        return self

    def __next__(self) -> Any:
        if self._iter is None:
            stack = self.stack = ExitStack()
            self.stack.__enter__()
            self._iter = cast(
                Iterator[Any],
                iter(
                    self.call.solve(
                        *self.args,
                        stack=stack,
                        cache_dependencies={},
                        nested=False,
                        **self.kwargs,
                    )
                ),
            )

        try:
            r = next(self._iter)
        except StopIteration:
            self.stack.__exit__(None, None, None)
            raise
        else:
            return r



from collections.abc import Generator, Iterable, Sequence
from contextlib import AsyncExitStack, ExitStack
from inspect import Parameter, unwrap
from typing import (
    TYPE_CHECKING,
    Any,
    Callable,
    Optional,
)

import anyio

from fast_depends._compat import ExceptionGroup
from fast_depends.library.model import CustomField
from fast_depends.library.serializer import OptionItem, Serializer
from fast_depends.utils import (
    async_map,
    is_async_gen_callable,
    is_coroutine_callable,
    is_gen_callable,
    run_async,
    solve_generator_async,
    solve_generator_sync,
)

if TYPE_CHECKING:
    from fast_depends.dependencies.provider import Key, Provider


class CallModel:
    __slots__ = (
        "call",
        "is_async",
        "is_generator",
        "params",
        "alias_arguments",
        "args_name",
        "positional_args",
        "kwargs_name",
        "keyword_args",
        "dependencies",
        "extra_dependencies",
        "custom_fields",
        "use_cache",
        "serializer",
        "dependency_provider",
    )

    alias_arguments: tuple[str, ...]

    @property
    def call_name(self) -> str:
        call = unwrap(self.call)
        return getattr(call, "__name__", type(call).__name__)

    @property
    def flat_params(self) -> list[OptionItem]:
        params = list(self.params)
        for d in map(
            self.dependency_provider.get_dependant,
            (*self.dependencies.values(), *self.extra_dependencies),
        ):
            for p in d.flat_params:
                if p.field_name not in (i.field_name for i in params):
                    params.append(p)
        return params

    def __init__(
        self,
        *,
        call: Callable[..., Any],
        serializer: Optional[Serializer],
        params: tuple[OptionItem, ...],
        use_cache: bool,
        is_async: bool,
        is_generator: bool,
        args_name: Optional[str],
        kwargs_name: Optional[str],
        dependencies: dict[str, "Key"],
        extra_dependencies: Iterable["Key"],
        keyword_args: list[str],
        positional_args: list[str],
        custom_fields: dict[str, CustomField],
        dependency_provider: "Provider",
    ):
        self.call = call
        self.serializer = serializer

        if serializer is not None:
            self.alias_arguments = serializer.get_aliases()
        else:  # pragma: no cover
            self.alias_arguments = ()

        self.args_name = args_name
        self.keyword_args = tuple(keyword_args or ())
        self.kwargs_name = kwargs_name
        self.positional_args = tuple(positional_args or ())

        self.use_cache = use_cache
        self.is_async = (
            is_async or
            is_coroutine_callable(call) or
            is_async_gen_callable(call)
        )
        self.is_generator = (
            is_generator or
            is_gen_callable(call) or
            is_async_gen_callable(call)
        )

        self.dependencies = dependencies or {}
        self.extra_dependencies = tuple(extra_dependencies or ())
        self.custom_fields = custom_fields or {}

        self.params = params
        self.dependency_provider = dependency_provider

    def _solve(
        self,
        /,
        *args: tuple[Any, ...],
        cache_dependencies: dict[Callable[..., Any], Any],
        **kwargs: dict[str, Any],
    ) -> Generator[
        tuple[
            Sequence[Any],
            dict[str, Any],
        ],
        Any,
        Any,
    ]:
        if self.use_cache and self.call in cache_dependencies:
            return cache_dependencies[self.call]

        kw: dict[str, Any] = {}
        for arg in self.keyword_args:
            if (v := kwargs.pop(arg, Parameter.empty)) is not Parameter.empty:
                kw[arg] = v

        if self.kwargs_name in self.alias_arguments:
            kw[self.kwargs_name] = kwargs
        else:
            kw.update(kwargs)

        for arg in self.positional_args:
            if arg not in kw:
                if args:
                    kw[arg], args = args[0], args[1:]
                else:
                    break

        keyword_args: Iterable[str]
        if self.args_name in self.alias_arguments:
            kw[self.args_name] = args
            keyword_args = self.keyword_args

        else:
            keyword_args = self.keyword_args + self.positional_args
            for arg in keyword_args:
                if not args:
                    break

                if arg not in self.dependencies and arg not in kw:
                    kw[arg], args = args[0], args[1:]

        solved_kw: dict[str, Any]
        solved_kw = yield args, kw

        args_: Sequence[Any]
        if self.serializer is not None:
            casted_options = self.serializer(solved_kw)
            solved_kw.update(casted_options)

        if self.args_name:
            args_ = (
                *map(solved_kw.pop, self.positional_args),
                *solved_kw.get(self.args_name, args),
            )
        else:
            args_ = ()

        kwargs_ = {
            arg: solved_kw.pop(arg)
            for arg in keyword_args
            if arg in solved_kw
        }
        if self.kwargs_name:
            kwargs_.update(solved_kw.get(self.kwargs_name, solved_kw))

        response = yield args_, kwargs_

        if not self.is_generator:
            response = self._cast_response(response)

        if self.use_cache:  # pragma: no branch
            cache_dependencies[self.call] = response

        return response

    def _cast_response(self, /, value: Any) -> Any:
        if self.serializer is not None:
            return self.serializer.response(value)
        return value

    def solve(
        self,
        /,
        *args: tuple[Any, ...],
        stack: ExitStack,
        cache_dependencies: dict[Callable[..., Any], Any],
        nested: bool = False,
        **kwargs: dict[str, Any],
    ) -> Any:
        cast_gen = self._solve(
            *args,
            cache_dependencies=cache_dependencies,
            **kwargs,
        )
        try:
            args, kwargs = next(cast_gen)
        except StopIteration as e:
            cached_value = e.value
            return cached_value

        for dep in map(self.dependency_provider.get_dependant, self.extra_dependencies):
            dep.solve(
                *args,
                stack=stack,
                cache_dependencies=cache_dependencies,
                nested=True,
                **kwargs,
            )

        for dep_arg, dep_key in self.dependencies.items():
            if dep_arg not in kwargs:
                kwargs[dep_arg] = self.dependency_provider.get_dependant(dep_key).solve(
                    *args,
                    stack=stack,
                    cache_dependencies=cache_dependencies,
                    nested=True,
                    **kwargs,
                )

        for custom in self.custom_fields.values():
            if custom.field:
                custom.use_field(kwargs)
            else:
                kwargs = custom.use(**kwargs)

        final_args, final_kwargs = cast_gen.send(kwargs)

        if self.is_generator and nested:
            response = solve_generator_sync(
                *final_args,
                call=self.call,
                stack=stack,
                **final_kwargs,
            )

        else:
            response = self.call(*final_args, **final_kwargs)

        try:
            cast_gen.send(response)
        except StopIteration as e:
            value = e.value

            if self.serializer is None or nested or not self.is_generator:
                return value

            else:
                return map(self._cast_response, value)

        raise AssertionError("unreachable")

    async def asolve(
        self,
        /,
        *args: tuple[Any, ...],
        stack: AsyncExitStack,
        cache_dependencies: dict[Callable[..., Any], Any],
        nested: bool = False,
        **kwargs: dict[str, Any],
    ) -> Any:
        cast_gen = self._solve(
            *args,
            cache_dependencies=cache_dependencies,
            **kwargs,
        )
        try:
            args, kwargs = next(cast_gen)
        except StopIteration as e:
            cached_value = e.value
            return cached_value

        for dep in map(self.dependency_provider.get_dependant, self.extra_dependencies):
            # TODO: run concurrently
            await dep.asolve(
                *args,
                stack=stack,
                cache_dependencies=cache_dependencies,
                nested=True,
                **kwargs,
            )

        for dep_arg, dep_key in self.dependencies.items():
            if dep_arg not in kwargs:
                kwargs[dep_arg] = await self.dependency_provider.get_dependant(dep_key).asolve(
                    *args,
                    stack=stack,
                    cache_dependencies=cache_dependencies,
                    nested=True,
                    **kwargs,
                )

        if self.custom_fields:
            custom_to_solve: list[CustomField] = []

            try:
                async with anyio.create_task_group() as tg:
                    for custom in self.custom_fields.values():
                        if custom.field:
                            tg.start_soon(run_async, custom.use_field, kwargs)
                        else:
                            custom_to_solve.append(custom)

            except ExceptionGroup as exgr:
                for ex in exgr.exceptions:
                    raise ex from None

            for j in custom_to_solve:
                kwargs = await run_async(j.use, **kwargs)

        final_args, final_kwargs = cast_gen.send(kwargs)

        if self.is_generator and nested:
            response = await solve_generator_async(
                *final_args,
                call=self.call,
                stack=stack,
                **final_kwargs,
            )
        else:
            response = await run_async(self.call, *final_args, **final_kwargs)

        try:
            cast_gen.send(response)
        except StopIteration as e:
            value = e.value

            if self.serializer is None or nested or not self.is_generator:
                return value

            else:
                return async_map(self._cast_response, value)

        raise AssertionError("unreachable")
